<?php

/*
 * ForumNewsFeedPlugin.class.php - <short-description>
 *
 * Copyright (C) 2007 - Marcus Lunzenauer <mlunzena@uos.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 */


require_once dirname(__FILE__) . '/vendor/feedcreator/feedcreator.class.php';
require_once 'lib/classes/Seminar.class.php';
require_once 'lib/forum.inc.php';
require_once 'lib/visual.inc.php';

/**
 * An Example Standard Plugin.
 *
 * @package     examples
 * @subpackage  standard
 *
 * @author    mlunzena
 * @copyright (c) Authors
 * @version   $Id$
 */

class ForumNewsFeedPlugin extends AbstractStudIPStandardPlugin {


  public static $formats = array(
                             'RSS0.91' => 'application/rss+xml',
                             'RSS1.0'  => 'application/xml',
                             'RSS2.0'  => 'application/xml',
                             'ATOM0.3' => 'application/atom+xml');


  function ForumNewsFeedPlugin() {

    parent::AbstractStudIPStandardPlugin();

    $this->setPluginiconname("img/plugin.png");

    $navigation =& new PluginNavigation();
    $navigation->setDisplayname(_("Forum-Newsfeed"));
    $this->setNavigation($navigation);

    $this->template_factory =
      new Flexi_TemplateFactory(dirname(__FILE__).'/templates');
  }


  function initialize() {

    if (isset($GLOBALS['SessSemName'][1])) {

      $this->setId($GLOBALS['SessSemName'][1]);

      # get token
      $token = $this->get_token();

      # set autodiscovery links
      $params = array('formats' => ForumNewsFeedPlugin::$formats,
                      'plugin'  => $this,
                      'token'   => $token);
      $GLOBALS['_include_additional_header'] =
        $this->template_factory->render('links', $params);
    }
  }


  function getPluginname() {
    return _("Forum-Newsfeed-Plugin");
  }


  function actionShow() {

    try {

      $template = $this->template_factory->open('subscribe');
      $params = array('formats' => ForumNewsFeedPlugin::$formats,
                      'plugin'  => $this,
                      'token'   => $this->get_token());
      echo $template->render($params);

    } catch (Exception $e) {
      echo $this->template_factory->render('error', array('exception' => $e));
    }
  }


  function get_token() {

    if (!isset($this->token)) {

      $db = new DB_Seminar();
      $db->queryf('SELECT newsfeed_token FROM seminare WHERE Seminar_id = "%s"',
                  $course_id = $this->getId());
      if (!$db->next_record()) {
        throw new Exception(dgettext(__CLASS__, "Veranstaltung existiert nicht."));
      }

      $token = $db->f('newsfeed_token');
      if (empty($token))
        $this->token = $this->update_token();
      else
        $this->token = $course_id.$token;
    }

    return $this->token;
  }


  function update_token() {
    $db = new DB_Seminar();
    $db->queryf('UPDATE seminare SET newsfeed_token = "%s" WHERE Seminar_id = "%s"',
                $token = md5(uniqid()), $course_id = $this->getId());
    if (!$db->affected_rows()) {
      throw new Exception(dgettext(__CLASS__, "Fehler beim Erneuern des Tokens"));
    }

    return ($this->token = $course_id.$token);
  }


  function actionRss() {
    ob_end_clean();


    list($course_id, $token) = str_split($_REQUEST['token'], 32);

    $this->setId($course_id);

    $format = $_REQUEST['format'];

    if ($this->get_token() !== $course_id.$token) {
      throw new Exception(dgettext(__CLASS__, "Falsches token."));
    }

    $course = new Seminar($course_id);

    $posts = $this->get_posts($course_id, 100);

    $rss = new UniversalFeedCreator();
    $rss->title          = _("Forum") . ': "' . $course->name . '"';
    $rss->description    = $course->subtitle;
    $rss->link           = $GLOBALS['ABSOLUTE_URI_STUDIP'] . 'seminar_main.php?auswahl=' . $course_id . '&amp;redirect_to=forum.php&amp;view=neue&amp;sort=age';
    $rss->syndicationURL = htmlspecialchars($_SERVER['REQUEST_URI']);

    foreach ((array)$posts as $post) {

      $description = quotes_decode(formatReady(forum_kill_edit($post['description']), TRUE, TRUE));

      $item = new FeedItem();
      $item->title = $post['name'];
      $item->link = $GLOBALS['ABSOLUTE_URI_STUDIP'] . 'seminar_main.php?auswahl=' . $course_id . '&redirect_to=forum.php&view=neue&sort=age';
      $item->description = $description;
      $item->date = (int) $post['mkdate'];
      $item->source = $GLOBALS['ABSOLUTE_URI_STUDIP'];
      $item->author = $post['author'];
      $rss->addItem($item);
    }

    $output = $rss->createFeed($format);

    header('Content-Type: '.$rss->contentType.'; charset='.$rss->encoding);
    echo $output;
    exit();
  }


  function get_posts($course_id, $limit) {
    $db = new DB_Seminar();
    $db->queryf('SELECT x.topic_id, x.name, x.author, x.mkdate, x.chdate as age, '.
                'y.name AS root_name, x.description, x.Seminar_id, '.
                'y.topic_id AS root_id, x.user_id '.
                'FROM px_topics x, px_topics y '.
                'WHERE x.root_id = y.topic_id '.
                'AND x.seminar_id = \'%s\' '.
                'GROUP by x.topic_id '.
                'ORDER BY age DESC '.
                'LIMIT %d',
                $course_id, $limit);

    $posts = array();
    while ($db->next_record()) {
      $posts[] = $db->Record;
    }
    return $posts;
  }
}
